/*
 * Copyright (c) 2011, Olivier MATZ <zer0@droids-corp.org>
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <aversive.h>
#include <aversive/queue.h>
#include <aversive/pgmspace.h>
#include <aversive/error.h>

#include <uart.h>
#include <scheduler.h>
#include <clock_time.h>
#include <parse.h>
#include <rdline.h>
#include <timer.h>

#include "xbee_neighbor.h"
#include "xbee_atcmd.h"
#include "xbee_stats.h"
#include "xbee_buf.h"
#include "xbee_proto.h"
#include "xbee.h"
#include "cmdline.h"
#include "callout.h"
#include "rc_proto.h"
#include "spi_servo.h"

extern volatile uint16_t global_ms;

#define NB_LOGS 4

/** ERROR NUMS */
#define E_USER_DEFAULT           194

#define LED1_ON()       sbi(PORTA, 2)
#define LED1_OFF()      cbi(PORTA, 2)

#define LED2_ON()       sbi(PORTA, 1)
#define LED2_OFF()      cbi(PORTA, 1)

#define LED3_ON()       sbi(PORTA, 0)
#define LED3_OFF()      cbi(PORTA, 0)

/* highest priority */
#define LED_PRIO           170
#define TIME_PRIO          160
#define SPI_PRIO           100 /* users of spi_servo must have lower prio */
/* lowest priority */

#define MAX_POWER_LEVEL 5
/* generic to all boards */
struct xbeeboard {
	/* command line interface */
	struct rdline rdl;
	char prompt[RDLINE_PROMPT_SIZE];

	/* log */
	uint8_t logs[NB_LOGS+1];
	uint8_t log_level;
	uint8_t debug;
};
extern struct xbeeboard xbeeboard;


/* used for timeouts and xbee rx callback */
struct xbee_ctx {
	int foreground;
	int channel;
	char atcmd_query[2];
	int (*func)(void *frame, unsigned len, void *arg);
	void *arg;
	struct callout timeout;
};

//extern cmdline_parse_ctx_t main_ctx;
extern struct xbee_dev *xbee_dev;
extern int xbee_raw;
extern int xbee_hexdump;
extern int xbee_debug;

extern struct callout_manager cm;


void bootloader(void);

void xbee_rx(struct xbee_dev *dev, int channel, int type,
	     void *frame, unsigned len, void *opaque);
int xbeeapp_send_atcmd(const char *atcmd_str,
		       void *param, unsigned param_len, int foreground,
		       int (*func)(void *frame, unsigned len, void *arg),
		       void *arg);
int xbeeapp_send_msg(uint64_t addr, void *data,
		     unsigned data_len, int foreground);

void xbee_stdin_enable(void);
void xbee_stdin_disable(void);

void xbee_load_timeout(struct xbee_ctx *ctx);
void xbee_unload_timeout(struct xbee_ctx *ctx);

void xbee_mainloop(void);
